clear all   
cap log close
set more off
set matsize 11000
set scheme s2mono

* setting locations of our baseline data - see qje_DataPreparation_final.do for construction of these
* for running a replication of this, need to change these file paths to reflect
* where your data is 
global dir /disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design
global baseline_data /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/final/bene070809_baseline
global baseline_wide /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/final/bene_0708_0809_wide
global baseline_placebo /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/final/bene70809_placebo
global baseline_full /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/final/bene070809_full
global baseline_2006 /disk/agedisk5/medicare.work/afinkels-DUA22559/kluender/mergefiles/finaldata/bene2006

cd ${dir} 

* create the location of an output folder to hold all the figures
cap mkdir output/Figures_and_tables

global inputs /disk/agedisk5/medicare.work/afinkels-DUA22559/abarnett/Analytic_Data/final
global output /disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/output/Figures_and_tables
global int /disk/agedisk3/medicare.work/finkels-DUA22559/tangb/contract_design/temp


* switches 
scalar prepare_data = 1 
scalar figure2 = 1
scalar figure3 = 1
scalar figure4 = 1
scalar figure5 = 1
scalar figure9 = 1  
scalar figureA2 = 1
scalar figureA3 = 1
scalar figureA4 = 1
scalar figureA8 = 1
scalar table1 = 1
scalar table2 = 1
scalar table3 = 1
scalar table4 =1 

* set how many bootstrap reps for the EM programs
local bstraps = 50 

** BELOW - WE define the programs that are used for the excess mass and probability of claim analysis

capture program drop ExcessMass
program define ExcessMass, eclass
	syntax [if]
	marksample touse

	tempname H hp hn tot hpos hneg neg_hat pos_hat bin bunch neg bintotal bin_2 bin_2_sq bin_sq bin_2_cb bin_cb bintotal_hat
	cap drop EM
preserve	
	gen `bin' = binsize*floor(icl_diff/binsize)
	gen `bunch' = `bin'>=-delta & `bin'<delta
	gen `neg' = `bin'<0
	bysort `bin': gen `bintotal' = _N
	keep `bin' `bintotal' `bunch' binsize delta `neg' R
	duplicates drop
	
	gen `bin_2' = `bin'+binsize
	sum `bintotal'
	local N = r(sum)
	
	gen `bin_2_sq' = `bin_2'^2
	gen `bin_sq' = `bin'^2
	gen `bin_2_cb' = `bin_2'^3
	gen `bin_cb' = `bin'^3

	nl (`bintotal' = ((`N'-2*R*{c})/(2*R^3))*(`bin_2_cb'-`bin_cb')+{b}*(`bin_2_sq'-`bin_sq')+{c}*(`bin_2'-`bin')) if `bin'<-delta
	predict `bintotal_hat'
	keep if `bunch'==1
	
	gen `neg_hat' = `bintotal_hat' if `neg'==1
		replace `neg_hat' = 0 if `neg_hat'==.
	gen `pos_hat' = `bintotal_hat' if `neg'==0
		replace `pos_hat' = 0 if `pos_hat'==.

	egen `hn' = sum(`neg_hat')
	egen `hp' = sum(`pos_hat')
	egen `H' = sum(`bintotal')

	gen double EM = (`H'-`hp'-`hn')/(`hp'+`hn')
	
	assert !missing(EM)

	keep EM
	duplicates drop

	mkmat EM, matrix(EMmat) rowprefix(EM)
	matrix EMT = EMmat'
	ereturn post EMT
restore
	
end

capture program drop ElasticityPolynomial2
program define ElasticityPolynomial2, eclass

	tempname H hp hn tot a b c quadratic_soln bb hpos hneg neg_hat pos_hat B bin_2 bin_2_sq bin_2_cb bin_sq bin_cb
	cap drop EM alpha
	
	keep bin bintotal binsize R bunch icl_amt delta neg preprice postprice size_of_plan
	duplicates drop

	gen `bin_2' = bin+binsize
	sum bintotal
	local N = r(sum)
	sum bintotal if bin<-delta & bin>=(-2*delta)
	local hn_temp = r(sum)
	sum bintotal if bin>=delta & bin<(2*delta)
	local hp_temp = r(sum)

	gen `bin_2_sq' = `bin_2'^2
	gen `bin_sq' = bin^2
	gen `bin_2_cb' = `bin_2'^3
	gen `bin_cb' = bin^3

	nl (bintotal = ((`N'-2*R*{c})/(2*R^3))*(`bin_2_cb'-`bin_cb')+{b}*(`bin_2_sq'-`bin_sq')+{c}*(`bin_2'-bin)) if bin<-delta
	predict bintotal_hat
	keep if bunch==1
	
	gen `neg_hat' = bintotal_hat if neg==1
		replace `neg_hat' = 0 if `neg_hat'==.
	gen `pos_hat' = bintotal_hat if neg==0
		replace `pos_hat' = 0 if `pos_hat'==.

	egen `hn' = sum(`neg_hat')
	egen `hp' = sum(`pos_hat')
	egen `H' = sum(bintotal)
	gen double `hpos' = `hp'/delta
	gen double `hneg' = `hn'/delta

	gen double EM = (`H'-`hp'-`hn')/(`hp'+`hn')
	gen double `B' = `H'-`hp'-`hn' 
	gen double `b' = `hpos'-`hneg'-(2*`B'/icl_amt) 
	gen double `quadratic_soln' = (-`b'+sqrt(`b'^2 + 4*`hneg'*`hpos'))/(2*`hneg')
	gen double alpha = log(`quadratic_soln')/log((2-preprice)/(2-postprice))
	gen double elasticity = (-alpha*preprice)/(2-preprice)

	assert !missing(EM)
	assert !missing(alpha)
	
	keep preprice postprice icl_amt EM alpha size_of_plan
	duplicates drop

	mkmat EM, matrix(EMmat) rowprefix(EM)
	mkmat alpha, matrix(alphamat) rowprefix(alpha)
	mkmat size_of_plan, matrix(size) rowprefix(wgt)
	matrix EMT = EMmat'
	matrix alphaT = alphamat'
	matrix sizeT= size'
	
	matrix `bb' = EMT,alphaT,sizeT
	ereturn post `bb'
	
end

capture program drop ProbClaim_any
program define ProbClaim_any, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_any)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

capture program drop ProbClaim_acute
program define ProbClaim_acute, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_acute)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

capture program drop ProbClaim_chr
program define ProbClaim_chr, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_chr)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

capture program drop ProbClaim_maint
program define ProbClaim_maint, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_maint)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end


capture program drop ProbClaim_nonmaint
program define ProbClaim_nonmaint, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_nonmaint)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

capture program drop ProbClaim_brand
program define ProbClaim_brand, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_brand)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

capture program drop ProbClaim_gen
program define ProbClaim_gen, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_gen)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

capture program drop ProbClaim_hedis
program define ProbClaim_hedis, eclass
	syntax [if]
	marksample touse

preserve	
	bysort bin: egen bin_mean = mean(nocl_01dec_hedis)
	bysort bin: gen freq = _N
	gen logy = log(bin_mean)
	
	keep bin bin_mean logy bunch freq
	duplicates drop
	
	qui: reg logy bin if bin<-500 [fw=freq]
		predict yhat
	gen bin_mean_hat = exp(yhat)

	gen yhat_cl = 1 - bin_mean_hat
	gen bin_mean_cl = 1 - bin_mean

	keep if bunch==1
	
	sum yhat_cl [fw=freq]
	gen hat = r(mean)
	sum bin_mean_cl [fw=freq]
	gen actual = r(mean)
	
	gen double Decline = (hat - actual)/hat

	keep Decline
	duplicates drop

	mkmat Decline, matrix(Declinemat) rowprefix(Decline)
	matrix DeclineT = Declinemat'
	ereturn post DeclineT
restore
	
end

if prepare_data == 1 { 
* Pull in wide dataset
use ${baseline_wide}, clear

* Keep only those who merge across either 07-08 or 08-09
keep if _merge_0708==3|_merge_0809==3 

*Create variables necessary for heterogeneity analysis

	*Some variables we might want to cut by
	//age
	xtile age_quart_y1 = age_y1, n(4)
	
	*Total conditions
	egen total_conditions=rowtotal(HCC*_y1), mi
	gen any_conditions=1 if total_conditions>0 & total_conditions~=.
	replace any_conditions=0 if total_conditions==0

	*Indicator for deductible
	gen ded=1 if ded_amt_y1>0 & ded_amt_y1<. & ded_amt_y2>0 & ded_amt_y2<.
	replace ded=0 if ded_amt_y1==0 & ded_amt_y2==0

	gen female=(sex_y1==2)

	//standard icl
	foreach y in 1 2 {
	gen standardicl_y`y' = 2400 if yr_y`y'==2007
	replace standardicl_y`y' = 2510 if yr_y`y'==2008
	replace standardicl_y`y'=2700 if yr_y`y'==2009
	replace standardicl_y`y'=2830 if yr_y`y'==2010
	gen has_standardicl_y`y'= (icl_amt_y`y'==standardicl_y`y')
	}
	keep if has_standardicl_y1==1 & has_standardicl_y2==1

	gen icl_type_y1 = (has_standardicl_y1==1)
	replace icl_type_y1=2 if has_standardicl_y1==0 & icl_amt_y1~=0 & icl_amt_y1~=.
	replace icl_type_y1=0 if (icl_amt_y1==. | icl_amt_y1==0)
		
	gen icl_type_y2 = (has_standardicl_y2==1)
	replace icl_type_y2=2 if has_standardicl_y2==0 & icl_amt_y2~=0 & icl_amt_y2~=.
	replace icl_type_y2=0 if (icl_amt_y2==. | icl_amt_y2==0)
	
	gen icl_sw=1 if (icl_type_y1==1 & icl_type_y2==1)
	replace icl_sw=2 if (icl_type_y1==0 & icl_type_y2==0)
	replace icl_sw=3 if (icl_type_y1==0 & icl_type_y2==1)
	replace icl_sw=4 if (icl_type_y1==1 & icl_type_y2==0)
	
	foreach phase in A_Ded B_PreICL C_ICL D_Cat {
		gen I_`phase' = finphas_clean_y2=="`phase'"
	}

	forv y = 1/2 {
		gen eoy_price_y`y' = .
			replace eoy_price_y`y' = plan_coin_ded_y`y' if finphas_clean_y`y'=="A_Ded"
			replace eoy_price_y`y' = plan_coin_preicl_y`y' if finphas_clean_y`y'=="B_PreICL"
			replace eoy_price_y`y' = plan_coin_gap_y`y' if finphas_clean_y`y'=="C_ICL"
			replace eoy_price_y`y' = plan_coin_cat_y`y' if finphas_clean_y`y'=="D_Cat"
	}

gen PreICL=(I_B_PreICL==1 | I_A_Ded==1)
	
*generate normalized spending
gen norm_total_y1=total_any_y1-icl_amt_y1
gen norm_total_y1_bin_sm=20*floor(norm_total_y1/20)
gen norm_total_y1_bin_m=50*floor(norm_total_y1/50)

*Generate cumulative monthly spending and "past kink" variable

gen m1_cum_y2=m1_total_any_y2
gen m1_past_kink_y2=1 if m1_cum_y2>=icl_amt_y2 & m1_cum_y2~=. & icl_amt_y2~=.
replace m1_past_kink_y2=0 if m1_cum_y2<icl_amt_y2 & m1_cum_y2~=. & icl_amt_y2~=.

foreach m of numlist 2/12  {
local n=`m'-1
gen m`m'_cum_y2=m`n'_cum_y2+m`m'_total_any_y2
gen m`m'_past_kink_y2=1 if m`m'_cum_y2>=icl_amt_y2 & m`m'_cum_y2~=. & icl_amt_y2~=.
replace m`m'_past_kink_y2=0 if m`m'_cum_y2<icl_amt_y2 & m`m'_cum_y2~=. & icl_amt_y2~=.
}



*Generate denominator for intertemporal spending 
	egen mean_m3_m6_y2=rowmean(m3_total_any_y2 m4_total_any_y2 m5_total_any_y2 m6_total_any_y2)

	foreach ty in chr acute gen brand maint nonmaint hedis nonhedis {
	egen mean_m3_m6_y2_`ty'=rowmean(m3_total_`ty'_y2 m4_total_`ty'_y2 m5_total_`ty'_y2 m6_total_`ty'_y2)	
	*generate new counterfactual
	sum m1_total_`ty'_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000
	gen cf_j_`ty' =r(mean)
	sum mean_m3_m6_y2_`ty' if norm_total_y1<=-500 & norm_total_y1>=-2000
	gen cf_m3_m6_`ty' =r(mean)
	gen cf_alt_`ty'=cf_j_`ty'/cf_m3_m6_`ty'
	gen n_m1_`ty'=m1_total_`ty'_y2
	}
	
*Generate person cuts
egen rxhccmiss=rowmiss(RXHCC*y1)
egen tot_rxhcc=rowtotal(RXHCC*y1) if rxhccmiss==0
gen tot_rxhcc_cen=tot_rxhcc
replace tot_rxhcc_cen=5 if tot_rxhcc_cen>5 & tot_rxhcc_cen<.

gen age_cat=66 if age_y1==66
replace age_cat=67 if age_y1==67
replace age_cat=68 if age_y1>=68 & age_y1<70
replace age_cat=70 if age_y1>=70 & age_y1<75
replace age_cat=75 if age_y1>=75 & age_y1<80
replace age_cat=80 if age_y1>=80 & age_y1<85
replace age_cat=85 if age_y1>=85 & age_y1<.

*Generate IS measure
	gen is_j_m3_m6=m1_total_any_y2/mean_m3_m6_y2
	gen excess_jan=m1_total_any_y2-mean_m3_m6_y2

*Generate adjust Y1 spending
gen total_any_adj_y1=total_any_y1+excess_jan
gen norm_total_adj_y1=total_any_adj_y1-icl_amt_y1
gen norm_total_adj_y1_bin_sm=20*floor(norm_total_adj_y1/20)
gen norm_total_adj_y1_bin_m=50*floor(norm_total_adj_y1/50)

*generate counterfactuals
	sum is_j_m3_m6 if norm_total_y1<=-500 & norm_total_y1>=-2000
	gen cf_j_m3_m6 =r(mean)
	gen rel_j_m3_m6 =(is_j_m3_m6 -cf_j_m3_m6 )/cf_j_m3_m6 
	
	sum is_j_m3_m6  if norm_total_y1<=-500 & norm_total_y1>=-2000 & ded==1
	gen cf_ded_j_m3_m6 =r(mean) if ded==1
	gen rel_ded_j_m3_m6 =(is_j_m3_m6 -cf_ded_j_m3_m6 )/cf_ded_j_m3_m6 if ded==1
	
	sum is_j_m3_m6  if norm_total_y1<=-500 & norm_total_y1>=-2000 & ded==0
	gen cf_noded_j_m3_m6 =r(mean) if ded==0
	gen rel_noded_j_m3_m6 =(is_j_m3_m6 -cf_noded_j_m3_m6 )/cf_noded_j_m3_m6  if ded==0
	
	*generate new counterfactual
	sum m1_total_any_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000
	gen cf_j =r(mean)
	sum mean_m3_m6_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000
	gen cf_m3_m6 =r(mean)
	gen cf_alt=cf_j/cf_m3_m6
	
	sum m1_total_any_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000  & ded==1
	gen cf_j_ded =r(mean)  if ded==1
	sum mean_m3_m6_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000  & ded==1
	gen cf_m3_m6_ded =r(mean) if  ded==1
	gen cf_alt_ded=cf_j_ded/cf_m3_m6_ded  if ded==1
	
	sum m1_total_any_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000  & ded==0
	gen cf_j_noded =r(mean) if ded==0
	sum mean_m3_m6_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000 & ded==0
	gen cf_m3_m6_noded =r(mean) if ded==0
	gen cf_alt_noded=cf_j_noded/cf_m3_m6_noded if ded==0
	
	gen n_m1=m1_total_any_y2
	gen n_j_m3_m6 =is_j_m3_m6 
	
gen m1_500=m1_total_any_y2 if norm_total_y1>=0 & norm_total_y1<=500
gen m1_u1500=m1_total_any_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000
gen m3_m6_500=mean_m3_m6_y2 if norm_total_y1>=0 & norm_total_y1<=500
gen m3_m6_u1500=mean_m3_m6_y2 if norm_total_y1<=-500 & norm_total_y1>=-2000

save ${int}/qje_intertemp.dta, replace

} 




** QJE RESUBMIT FIGURE 2 
if figure2 == 1 {
use ${baseline_data}, clear

gen sample = yr==2008 & totcost<6500
tab sample
keep if sample

gen freq = 1
gen bin = 100*floor(totcost/100)
replace bin = bin+50

collapse (sum) freq, by(bin)

egen total = total(freq)
gen percent = 100*freq/total
sum total 
local N= r(max) 

graph twoway bar percent bin, barw(100) col(gs7) ///
		ylabel(0(1)11) xlabel(0(500)6500) ///
		xtitle("2008 spending distribution") ///
		ytitle("freq")  ///
		title("total annual expenditure" "N=`N'")
graph export ${output}/Figure2.png, replace

outsheet percent bin using ${output}/Figure2.csv, c replace
}

** QJE RESUBMIT FIGURE 3.
if figure3 == 1 {

use ${baseline_data}, clear

gen sample =  totcost>1500 & totcost<3500
tab sample
keep if sample
tab yr, mi

gen freq = 1
gen totcost_bin = 20*floor(totcost/20)

collapse (sum) freq, by(totcost_bin icl_amt yr)

bys yr: egen yr_obs = total(freq)
egen total = total(freq)
gen pct_freq = freq/yr_obs
forval year == 2007/2009 { 
	sum yr_obs if yr == `year' 
	local n_`year' = r(mean) 
	sum icl_amt if yr == `year' 
	local icl_`year' = r(mean)
}
sum total
local n = r(mean) 

twoway (scatter freq totcost_bin if yr == 2007) ///
	(scatter freq totcost_bin if yr == 2008) ///
	(scatter freq totcost_bin if yr == 2009), /// 
	xline(`icl_2007') xline(`icl_2008') xline(`icl_2009') ///
	title("total n=`n', 2007 n:`n_2007'" "2008 n: `n_2008' , 2009 n: `n_2009'") 
graph export ${output}/Figure3.png, replace 

outsheet * using ${output}/Figure3.csv, c replace

} // end figure3


** QJE RESUBMIT FIGURE 4.
if figure4 == 1 {

use ${baseline_data}, clear

gen sample = icl_diff>=-2000 & icl_diff<2000 
tab sample
keep if sample

local binsize 20
local delta 200

gen bin = `binsize'*floor(icl_diff/`binsize')
bysort bin: gen bintotal = _N

keep bin bintotal 
duplicates drop

gen bin_2 = bin+`binsize'
sum bintotal
local N = r(sum)

gen bin_2_sq = bin_2^2
gen bin_sq = bin^2
gen bin_2_cb = bin_2^3
gen bin_cb = bin^3
gen R = 2000
gen delta = 200

nl (bintotal = ((`N'-2*R*{c})/(2*R^3))*(bin_2_cb-bin_cb)+{b}*(bin_2_sq-bin_sq)+{c}*(bin_2-bin)) if bin<-delta
predict bintotal_hat

* generate the percentages
egen tot_obs = total(bintotal)
gen pct_freq = bintotal/tot_obs 
gen cf_pct_freq = bintotal_hat/tot_obs

graph twoway (scatter pct_freq bin) (line cf_pct_freq bin), ///
	title("magnitude of exciess mass" "N = `N'") ///
	xlabel(-2000(500)2000) 
graph export ${output}/Figure4.png, replace 

outsheet pct_freq cf_pct_freq bintotal bintotal_hat bin using ${output}/Figure4.csv, c replace

} // end figure4


* QJE RESUBMIT FIGURE 5
if figure5 == 1 {

	//Prob of 9-12 montlhy claims w CF
	foreach m of numlist 9/12 {

	use ${baseline_data}.dta, clear
	
	*generate claim probability in each month
	gen claim_m`m'=1 if m`m'_total_any>0 & m`m'_total_any<.
	replace claim_m`m'=0 if m`m'_total_any==0

	*limit to within 2000 of icl
	gen norm_total=total_any-icl_amt
	gen norm_total_bin_sm=20*floor(norm_total/20)

	keep if norm_total_bin_sm<=2000 & norm_total_bin_sm>=-2000
	count
	local N= r(N) 

	*collapse, keeping track of size of each spending bin
	keep norm_total_bin_sm claim_m`m'
	gen freq=1
	collapse (mean) claim_m`m' (sum) freq , by(norm_total_bin_sm)

	*generate counterfactual
	gen no_claim=1-claim_m`m'
	gen logy = log(no_claim)
	qui: reg logy norm_total_bin_sm if norm_total_bin_sm<-500 [fw=freq]
	predict yhat_e2
	gen yhat_alt2 = exp(yhat_e2)
	gen yhat = 1-yhat_alt2


	*graph of probaility of monthly claim
	twoway (scatter claim_m`m' norm_total_bin_sm, msize(small) ///
		xtitle("Spending Relative to ICL") ytitle("Prob of Claim in Month `m'") xlabel(#10)) ///
		(line yhat norm_total_bin_sm, lstyle(dash)), title("Monthly timing graphs" "N:`N'") legend(off)
	graph export ${output}/Figure5_m`m'.png, replace 
	outsheet using ${output}/Figure5_m`m'.csv, comma replace

	}
	
}

* QJE RESUBMIT FIGURE 9
if figure9 == 1 { 
use ${int}/qje_intertemp.dta, clear
* PANEL A 
	collapse (mean) is_j_m3_m6  cf_j_m3_m6  rel_j_m3_m6 mean_m3_m6_y2 m1_total_any_y2 cf_alt (count) n_j_m3_m6 n_m1, by(norm_total_y1_bin_m)
	keep if norm_total_y1<=2000 & norm_total_y1>=-2000
	gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2
	gen rel_j_m3_m6_smooth=(is_j_m3_m6_smooth -cf_alt )/cf_alt


		egen Nj_m3_m6 =total(n_j_m3_m6 )
		sum Nj_m3_m6 
		local N=r(max)

		egen Nm1 =total(n_m1 )
		sum Nm1
		local N2=r(max)

		twoway 	(bar is_j_m3_m6_smooth norm_total_y1_bin, barw(50) col(gs7)) ///
			(line cf_alt  norm_total_y1_bin, lc(black)), ///
			title("CF based on -500 to -2000 uniform") ylabel(0.8(0.2)1.4) ///
			xtitle("Year T Spending Relative to ICL") ///
			ytitle("")  ///
			legend(off) caption("Number of observations = `N2'.")
		
		graph export "${output}/Figure9a.png", replace
		
		export delim is_j_m3_m6_smooth cf_alt norm_total_y1_bin using ${output}/Figure9a.csv, replace


* PANEL B

use ${int}/qje_intertemp.dta, clear
gen freq = 1 


	collapse (count) freq, by(norm_total_y1_bin_m)
	keep if norm_total_y1<=2000 & norm_total_y1>=-2000

	egen total_count = total (freq)
	gen percent_bin = freq / total_count 
		sum total_count
		local N = r(max)
		di "`N'"
		
	twoway 	(scatter percent_bin norm_total_y1_bin_m,  ///
			xtitle("Adjusted Year T Spending Relative to ICL") ///
			xlabel(-2000(500)2000) ///
			ytitle("Normalized Frequency")  ///
			legend(off) caption("Number of observations = `N'."))
			
	graph export "${output}/Figure9b.png", replace

export delim  norm_total_y1_bin_m freq percent_bin using "${output}/Figure9b.csv", replace
}



* QJE RESUBMIT FIGURE A2
if figureA2 == 1 {

use ${baseline_placebo}, clear

gen sample = icl_diff>=-1000 & icl_diff<1000 & !nogap & icl_amt==2100
tab sample
keep if sample

gen icl_diff_bin = 20*floor(icl_diff/20)
gen freq = 1

collapse (sum) freq, by(icl_diff_bin icl_amt)

egen total = total(freq)
gen percent = 100*freq/total
gen standard_icl_2007 = 2400
gen standard_icl_2008 = 2510

sum total
local n= r(mean)

graph twoway (bar percent icl_diff_bin, barw(15)), ylabel(0(0.5)2) xlabel(-1000(200)1000) ///
	yline(2400) yline(2510) title("non-standard ICLs" "n=`n'")
graph export ${output}/FigureA2.png, replace

outsheet icl_amt icl_diff_bin freq total percent using ${output}/FigureA2.csv, c replace

} // figureA2


* QJE RESUBMIT FIGURE A3 
if figureA3 == 1 { 
use ${baseline_data}, clear

gen sample = has_standardded // note this used to be ded_amt>100 & ded_amt!=.
tab sample
keep if sample

gen sample2 = ded_diff>=-250 & ded_diff<250
tab sample2
keep if sample2

gen ded_diff_bin = 10*floor(ded_diff/10)
gen freq = 1

collapse (sum) freq, by(ded_diff_bin)

egen total = total(freq)
gen percent = 10*freq/total
sum total
local N = r(mean) 


graph twoway bar percent ded_diff_bin, barw(10) col(gs7) ///
		ylabel(0(0.05)0.4) xlabel(-250(50)200) ///
		xtitle("Spending Relative to ICL") ///
		ytitle("Normalized freq")  ///
		title("Spending around ded" "N=`N'")
graph export ${output}/FigureA2.png, replace

outsheet ded_diff_bin percent using ${output}/FigureA2.csv, c replace
}


* QJE RESUBMIT FIGURE A4
if figureA4 == 1{ 


use ${baseline_data}.dta, clear

count
keep if icl_diff<2000 & icl_diff>=-2000 
bysort plan yr: gen size_of_plan = _N
keep if size_of_plan>=1000 & size_of_plan!=.
count

gen delta = 200
gen binsize = 20
gen preprice = .3
gen postprice = 1
	replace preprice = round(plan_coin_preicl, .001)
	replace postprice = round(plan_coin_gap, .001)
gen R = 2000
drop if preprice==postprice
gen bin = binsize*floor(icl_diff/binsize)
gen bunch = bin>=-delta & bin<delta
gen neg = bin<0
bysort bin icl_amt preprice postprice: gen bintotal = _N
gen kink = postprice - preprice
keep bin bintotal binsize R bunch icl_amt delta neg preprice postprice size_of_plan
duplicates drop

sort icl_amt preprice postprice
egen group = group(icl_amt preprice postprice size_of_plan)
	
levelsof group, local(groups)
foreach group of local groups {
	
	preserve
	keep if group==`group'
	ElasticityPolynomial2
	tempfile egroup`group'
	save `egroup`group''
	restore
		
} // foreach group
	
	use `egroup1', clear
	foreach group of local groups {
		append using `egroup`group''
	} // end foreach group
	
	gen kink = postprice - preprice

egen total = total(size_of_plan)

reg EM kink [pw=size_of_plan]

local r2= e(r2)

local slope = _b[kink]
local constant = _b[_cons] 
gen fitted = `constant' + `slope' * kink 
sum total
local N = r(mean) 

gen test = size_of_plan/total

graph twoway (scatter EM kink [w=test] if EM < 1, msymbol(circle_hollow) msize(tiny)) ///
	(line fitted kink, msize(small)), ///
	ytitle("EM around kink") xtitle("Cost-sharing change at kink") ///
	title("EM by plan" "slope =`slope', r2 = `r2' N = `N'")
graph export ${output}/Figure6.png, replace

outsheet EM kink size_of_plan postprice preprice fitted using ${output}/Figure6.csv, c replace

}



* QJE RESUBMIT FIGURE A8  
if figureA8 == 1 { 
	use ${int}/qje_intertemp.dta, clear

	drop if ded==.

	collapse (mean) is_j_m3_m6  cf_j_m3_m6  rel_j_m3_m6 mean_m3_m6_y2  m1_total_any_y2 cf_alt* (count) n_j_m3_m6 n_m1, by(norm_total_y1_bin_m ded)
	keep if norm_total_y1<=2000 & norm_total_y1>=-2000
	replace cf_alt=cf_alt_ded if ded==1
	replace cf_alt=cf_alt_noded if ded==0

	gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2
	gen rel_j_m3_m6_smooth=(is_j_m3_m6_smooth -cf_alt )/cf_alt

	*Graphs

		bysort ded: egen Nm1=total(n_m1)
		sum Nm1 if ded==1
		local N_ded=r(max)
		sum Nm1 if ded==0
		local N_noded=r(max)
	
	twoway 	(bar is_j_m3_m6_smooth norm_total_y1_bin if ded==0, barw(50) col(gs7)) ///
		(line cf_alt norm_total_y1_bin if ded==0, lc(gs7)) ///
		(bar is_j_m3_m6_smooth norm_total_y1_bin if ded==1, barw(50) col(black)) ///
		(line cf_alt norm_total_y1_bin if ded==1, lc(black)), ///
		title("Relative Jan spending by plan type""Ded n=`N_ded', noded n = `N_noded'") ylabel(0.8(0.1)1.4) ///
		xtitle("Year T Spending Relative to ICL") ///
		ytitle("")  ///
		legend(off)
	graph export ${output}/FigureA8.png, replace

	export delim is_j_m3_m6_smooth cf_alt is_j_m3_m6_smooth norm_total_y1_bin ded using ${output}/FigureA8.csv, replace
	
}



* QJE RESUBMIT TABLE1 
if table1 == 1 { 

use ${baseline_full}, clear
putexcel set ${output}/Table1.xls, replace sheet("full sample", replace) 

	matrix table1a = J(2,1,.) 
	matrix rownames table1a = "Obs in bene-years" "Unique beneficiaries" 

	count
	local obs = r(N) 
	mat table1a[1,1] = `obs' 
	unique bene_id
	local unique_n = r(sum) 
	mat table1a[2,1] =  `unique_n' 
	
	putexcel A1 = matrix(table1a, names) 

	gen zero_spend = totcost==0
	gen zero_oopspend = oopspend==0

	estpost tabstat age female zero_oopspend zero_spend, listwise statistics(mean sd)
	matrix table1b = e(age)\e(female)\e(zero_oopspend)\e(zero_spend) 
	matrix rownames table1b = "age" "female" "percent with 0 OOP" "percent with 0 spend" 
	mat colnames table1b = "mean" "Sd" 
		putexcel A4 = matrix(table1b, names) 

	estpost tabstat oopspend totcost, listwise statistics(mean sd p25 p50 p75 p90)
	matrix table1c=e(oopspend)\e(totcost) 
	matrix rownames table1c = "oop spend" "any spend" 
	mat colnames table1c = "mean" "Sd" "25pct" "50pct" "75pct" "90pct" 
	putexcel A9 = matrix(table1c, names) 

putexcel clear 

putexcel set ${output}/Table1.xls, modify sheet("baseline sample",replace) 

	use ${baseline_data}, clear

	matrix table1a = J(2,1,.) 
	matrix rownames table1a = "Obs in bene-years" "Unique beneficiaries" 

	count
	local obs = r(N) 
	mat table1a[1,1] = `obs' 
	unique bene_id
	local unique_n = r(sum) 
	mat table1a[2,1] =  `unique_n' 
	
	putexcel A1 = matrix(table1a, names) 

	gen zero_spend = totcost==0
	gen zero_oopspend = oopspend==0

	estpost tabstat age female zero_oopspend zero_spend riskscore_CE, listwise statistics(mean sd)
	matrix table1b = e(age)\e(female)\e(riskscore_CE)\e(zero_oopspend)\e(zero_spend)
	matrix rownames table1b = "age" "female" "risk score" "percent with 0 OOP" "percent with 0 spend" 
	mat colnames table1b = "mean" "Sd" 
		putexcel A4 = matrix(table1b, names) 

	estpost tabstat oopspend totcost, listwise statistics(mean sd p25 p50 p75 p90)
	matrix table1c=e(oopspend)\e(totcost) 
	matrix rownames table1c = "oop spend" "any spend" 
	mat colnames table1c = "mean" "Sd" "25pct" "50pct" "75pct" "90pct" 
	putexcel A10 = matrix(table1c, names) 

putexcel clear 
}


*QJE RESUBMIT TABLE 2 
if table2 ==1{ 

use ${baseline_data}, clear

gen freq = 1

drop plan_group
gen plan_group=""
replace plan_group = "ded" if ded_amt>0 & ded_amt!=.
replace plan_group = "noded" if ded_amt==0

gen gap_coin_nogapcov = plan_coin_gap if nogapcov
gen gap_coin_somegapcov = plan_coin_gap if nogapcov==0

collapse (sum) freq (mean) ded_amt plan_coin_ded has_standardicl icl_amt plan_coin_preicl nogapcov gap_coin_nogapcov gap_coin_somegapcov oopt_amt plan_coin_cat, by(plan_group)

gen gapcov = 1-nogapcov
outsheet plan_group freq ded_amt plan_coin_ded icl_amt plan_coin_preicl gapcov gap_coin_nogapcov gap_coin_somegapcov oopt_amt plan_coin_cat using ${output}/Table2.csv, replace c

}
* QJE RESUBMIT TABLE 3
if table3 == 1 {

mat table3 = J(24,4,.) 
matrix colnames table3 ="Share of sample" "Share of spending" "Excess Mass" "Excess January Spending"  


** OVERALL

use ${baseline_data}, clear
	matrix table3[1,1] = 100
	matrix table3[1,2] = 100

	gen delta = 200
	gen binsize = 20
	gen R= 2000

	keep if icl_diff<2000 & icl_diff>=-2000
	count
	di "Sample size for table8: r(N)" 
	di "Excess mass" 
	bootstrap, reps(`bstraps'): ExcessMass
	matrix table3[1,3] = _b[EM1] 

use ${int}/qje_intertemp.dta, clear
	collapse (mean) mean_m3_m6_y2  m1_total_any_y2 (count) n_m1, by(norm_total_y1_bin_m)
	keep if norm_total_y1<=200 & norm_total_y1>=-200

	gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2

	local i = 0
	sum is_j_m3_m6_smooth[fw = n_m1] 
	local is_kink_avg_`all' = r(mean) 
	matrix table3[1,4] = `is_kink_avg_`all'' 
	local rownames = "All"
	
	
matrix rownames table3 = `rownames' 
mat list table3
	

** BY AGE **

use ${baseline_data}, clear

gen delta = 200
gen binsize = 20
gen R= 2000


gen age_bin = 66 if age==66
	replace age_bin = 67 if age==67
	replace age_bin = 68 if age==68|age==69
	replace age_bin = 70 if age>=70 & age<=74
	replace age_bin = 75 if age>=75 & age<=79
	replace age_bin = 80 if age>=80 & age<=84
	replace age_bin = 85 if age>=85 & age!=.
levelsof age_bin, local(age_bins)	
	
sum totcost
local total = r(sum)
di "`total'"

local row_num = 2 
foreach age of local age_bins {
	preserve
		di "`age'" 
		if `age' == 66 { 
			local bin_age = "66"
		}
		if `age' == 67 { 
			local bin_age = "67"
		}
		if `age' == 68 { 
			local bin_age = "68-69" 
		}
		if `age' == 70 { 
			local bin_age = "70-74"
		}
		if `age' == 75 { 
			local bin_age = "75-79"
		}
		if `age' == 80 { 
			local bin_age = "80-84" 
		}
		if `age' == 85 { 
			local bin_age = "85-plus" 
		}
		local rownames `rownames' "`bin_age'" 
		
		gen age_`age' = age_bin == `age'
		di "share of sample:" 
		count 
		local tot = r(N) 
		count if age_`age' == 1 
		local age_count = r(N)
		local sample_share = 100*`age_count'/`tot' 
		matrix table3[`row_num',1] = `sample_share' 
		
		tab age_`age'
		keep if age_`age'
		sum totcost
		
		local share = r(sum)
		local spending_share = 100*`share'/`total'
		matrix table3[`row_num',2] = `spending_share' 
		di "Spending share: `spending_share'"
		
		keep if icl_diff<2000 & icl_diff>=-2000
		di "Excess mass" 
		bootstrap, reps(`bstraps'): ExcessMass
		matrix table3[`row_num',3] = _b[EM1] 
		
	use ${int}/qje_intertemp.dta, clear
	
	keep if age_cat == `age' 
	
	collapse (mean) mean_m3_m6_y2  m1_total_any_y2 (count) n_m1, by(norm_total_y1_bin_m age_cat)
	keep if norm_total_y1<=200 & norm_total_y1>=-200
	
	
	gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2
		
	sum is_j_m3_m6_smooth[fw = n_m1] 
	local is_kink_avg_`age' = r(mean) 
	matrix table3[`row_num',4] = `is_kink_avg_`age'' 

	restore
	local row_num = `row_num' + 1 
}	
	
	matrix rownames table3 = `rownames' 
	mat list table3 
	
	
** BY YEAR ** 
use ${baseline_2006}, clear

* KEEP ONLY THE BENES OBSERVED OVER ALL FOUR YEARS:
keep if partd_joinmonth==1
keep bene_id yr
tempfile benes_2006
save `benes_2006'

forv year=2007/2009 {
	use ${baseline_data}, clear
	keep if yr==`year'
	keep bene_id yr
	tempfile benes_`year'
	save `benes_`year''
}

use `benes_2006', clear

merge 1:1 bene_id using `benes_2007', gen(_benes_0607) keep(match)
merge 1:1 bene_id using `benes_2008', gen(_benes_08) keep(match)
merge 1:1 bene_id using `benes_2009', gen(_benes_09) keep(match)

keep bene_id
tempfile bene_ids
save `bene_ids'

use ${baseline_2006}, clear

merge 1:1 bene_id using `bene_ids', keep(match) gen(_match_beneids)

gen delta = 200
gen binsize = 20
gen R =2000

keep if icl_diff<2000 & icl_diff>=-2000
count
di "Year 2006" 
local rownames `rownames' "2006" 
bootstrap, reps(`bstraps'): ExcessMass
matrix table3[`row_num',3] = _b[EM1]
local row_num = `row_num' + 1 

use ${baseline_data}, clear

merge m:1 bene_id using `bene_ids', keep(match) gen(_match_beneids)

gen delta = 200
gen binsize = 20
gen R = 2000

foreach yr in 2007 2008 2009 {
	preserve
		di "year `yr'"
		local rownames `rownames' "`yr'" 
		keep if icl_diff<2000 & icl_diff>=-2000 & yr==`yr'
		count
		bootstrap, reps(`bstraps'): ExcessMass
		matrix table3[`row_num',3] = _b[EM1]	

		if `yr' == 2007 | `yr' == 2008 { 
			use ${int}/qje_intertemp.dta, clear
			keep if yr_y1 ==`yr' 
			//keep within window 
			keep if norm_total_y1<=200 & norm_total_y1>=-200
			
			
			keep mean_m3_m6_y2 m1_total_any_y2 yr_y1 yr_y2 n_m1 norm_total_y1_bin_m 
			
			collapse (mean) mean_m3_m6_y2  m1_total_any_y2 (count) n_m1, by(norm_total_y1_bin_m yr_y1)
		
			//generate the I/S measure 
			gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2
			sum is_j_m3_m6_smooth[fw = n_m1] 
			local is_kink_avg_`yr' = r(mean) 
			matrix table3[`row_num',4] = `is_kink_avg_`yr'' 	
		} 
		
			local row_num = `row_num' + 1

	restore
}

matrix rownames table3 = `rownames' 
mat list table3


** BY GENDER **

use ${baseline_data}, clear

gen delta = 200
gen binsize = 20
gen R= 2000

di "Share of sample" 
tab female, mi

foreach gender in 0 1 {
	preserve
		di "female=`gender'"
		if `gender' == 1 { 
			local gender_type = "female" 
		} 
		if `gender' == 0 { 
			local gender_type = "male" 
		}
		local rownames `rownames' "`gender_type'" 
		count 
		local tot_count = r(N) 
		count if female == `gender' 
		local num_sample = r(N) 
		local share_sample = 100*`num_sample'/`tot_count' 
		matrix table3[`row_num',1] = `share_sample' 	

		
		keep if female==`gender'
		
		sum totcost
		local share = r(sum)
		local spending_share = 100*`share'/`total'
		matrix table3[`row_num',2] = `spending_share' 
		
		di "Spending share: `spending_share', female=`gender'"
		
		keep if icl_diff<2000 & icl_diff>=-2000
		di "Excess Mass" 
		bootstrap, reps(`bstraps'): ExcessMass
		matrix table3[`row_num',3] = _b[EM1]	

		use ${int}/qje_intertemp.dta, clear
		if `gender' == 0 { 
			local gender_code = 1
		}
		if `gender' == 1 { 
			local gender_code = 2 
		}
			keep if sex_y1 ==`gender_code'
	
		collapse (mean) mean_m3_m6_y2  m1_total_any_y2 (count) n_m1, by(norm_total_y1_bin_m sex_y1)
		keep if norm_total_y1<=200 & norm_total_y1>=-200
		gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2
		
			
			sum is_j_m3_m6_smooth[fw = n_m1] 
			local is_kink_avg_`gender' = r(mean) 
			matrix table3[`row_num',4] = `is_kink_avg_`gender'' 
	restore
	local row_num = `row_num' + 1 
}

matrix rownames table3 = `rownames' 
mat list table3


** BY CHRONIC CONDITION **

use ${baseline_data}, clear

gen delta = 200
gen binsize = 20
gen R= 2000

egen rxhcc_total = rowtotal(RXHCC*)
	replace rxhcc_total = 5 if rxhcc_total>5
di "Share of sample" 
tab rxhcc_total, mi

foreach hcc in 0 1 2 3 4 5 {
	preserve
		di "# of HCC: `hcc'" 
		local rownames `rownames' "HCC:`hcc'" 
		count 
		local tot_count = r(N) 
		count if rxhcc_total==`hcc'
		local num_sample = r(N) 
		local share_sample = 100*`num_sample'/`tot_count' 
		matrix table3[`row_num',1] = `share_sample' 
	
		keep if rxhcc_total==`hcc'
		sum totcost
		local share = r(sum)
		local spending_share = 100*`share'/`total'
		di "Spending share: `spending_share', #hccs=`hcc'"
		matrix table3[`row_num',2] = `spending_share' 

		keep if icl_diff<2000 & icl_diff>=-2000
		di "Excess mass" 
		bootstrap, reps(`bstraps'): ExcessMass
		matrix table3[`row_num',3] = _b[EM1]	
	
	use ${int}/qje_intertemp.dta, clear
	keep if tot_rxhcc_cen == `hcc' 

	
	collapse (mean) mean_m3_m6_y2  m1_total_any_y2 (count) n_m1, by(norm_total_y1_bin_m tot_rxhcc_cen)
	keep if norm_total_y1<=200 & norm_total_y1>=-200
	drop if tot_rxhcc_cen==.

	gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2
	sum is_j_m3_m6_smooth[fw = n_m1] 
	local is_kink_avg_`hcc' = r(mean) 
	matrix table3[`row_num',4] = `is_kink_avg_`hcc'' 

	restore
	local row_num = `row_num' + 1 
	
}

matrix rownames table3 = `rownames' 
mat list table3


** BY RISK SCORE QUARTILE

use ${baseline_data}, clear

gen delta = 200
gen binsize = 20
gen R= 2000

drop if riskscore_CE==.
sum totcost
local total = r(sum)
xtile rx_rsq = riskscore_CE, n(4)
di "Share of sample:" 
tab rx_rsq, mi


foreach rsq in 1 2 3 4 {
	preserve
		di "RSQ: `rsq'"
		local rownames `rownames' "RSQ:`rsq'" 
		
		count 
		local tot_count = r(N) 
		count if rx_rsq==`rsq'
		local num_sample = r(N) 
		local share_sample = 100*`num_sample'/`tot_count' 
		matrix table3[`row_num',1] = `share_sample' 
		
		keep if rx_rsq==`rsq'
		sum totcost
		local share = r(sum)
		local spending_share = 100*`share'/`total'
		di "Spending share: `spending_share', rsq=`rsq'"
		matrix table3[`row_num',2] = `spending_share' 

		keep if icl_diff<2000 & icl_diff>=-2000
		di "Excess Mass:" 
		bootstrap, reps(`bstraps'): ExcessMass
		matrix table3[`row_num',3] = _b[EM1]	

	use ${int}/qje_intertemp.dta, clear

	
	collapse (mean) mean_m3_m6_y2  m1_total_any_y2 (count) n_m1, by(norm_total_y1_bin_m rsq_rx_y1)
	keep if norm_total_y1<=200 & norm_total_y1>=-200
	drop if rsq_rx_y1==.

	gen is_j_m3_m6_smooth=m1_total_any_y2/mean_m3_m6_y2

	keep if rsq_rx_y1 == `rsq' 
	sum is_j_m3_m6_smooth[fw = n_m1] 
	local is_kink_avg_`rsq' = r(mean) 
	matrix table3[`row_num',4] = `is_kink_avg_`rsq'' 

	restore
	local row_num = `row_num' + 1
}

matrix rownames table3 = `rownames' 
mat list table3

putexcel A1 = matrix(table3, names) using "${output}/Table3.xls", ///
		sheet("table3", replace) replace 		

} 


* QJE RESUBMIT TABLE 4
if table4 == 1 {
local bstraps = 2 
mat table4 = J(8,7,.) 
matrix colnames table4 ="Percent of purchase" "Percent of Spending" "Actual P(Dec)" ///
	"Predicted P(Dec)" "% decrease in purchase prob" "SE on % purchase" "Excess January Spending" 


use ${baseline_data}, clear

local row_num = 1 

sum totcost
local total = r(sum)
sum clm_any
local total_clm = r(sum)


foreach clm in any chr acute maint nonmaint brand gen hedis {
	local rownames `rownames' "`clm'" 
	sum clm_`clm'
	local clm_share = r(sum)
	local claim_share = 100*`clm_share'/`total_clm'
	mat table4[`row_num',1] = `claim_share' 
	
	sum total_`clm'
	local share = r(sum)
	local spending_share = 100*`share'/`total'
	mat table4[`row_num',2] = `spending_share' 
	di "`clm' Claim Share = `claim_share' Spend Share = `spending_share'"
	
	local row_num = `row_num' + 1 
}


mat  rownames table4 = `rownames' 

keep if icl_diff<2000 & icl_diff>=-2000
gen bunch = 0
	replace bunch = 1 if icl_diff>=-200 & icl_diff<200
gen binsize = 20
gen bin = binsize*floor(icl_diff/binsize)

local row_num = 1 
foreach clm in any acute chr maint nonmaint brand gen hedis {
	di "`clm'"
		
	preserve	
		bysort bin: egen bin_mean = mean(nocl_01dec_any)
		bysort bin: gen freq = _N
		gen logy = log(bin_mean)
		
		keep bin bin_mean logy bunch freq
		duplicates drop
		
		qui: reg logy bin if bin<-500 [fw=freq]
			predict yhat
		gen bin_mean_hat = exp(yhat)

		gen yhat_cl = 1 - bin_mean_hat
		gen bin_mean_cl = 1 - bin_mean

		keep if bunch==1
		
		sum bin_mean_cl [fw=freq]
		local actual = r(mean)
			mat table4[`row_num',3] = `actual' 
		sum yhat_cl [fw=freq]
		local hat = r(mean)
			mat table4[`row_num',4] = `hat'
		
		* I/S Column 7 
		use ${int}/qje_intertemp.dta, clear

		collapse (mean) mean_m3_m6_y2*  m1_total_*_y2 (count) n_m1*, by(norm_total_y1_bin_m)

		//keep within our window at the kink
		keep if norm_total_y1<=200 & norm_total_y1>=-200

		if "`clm'" == "any" { 
			gen is_j_m3_m6_smooth_any = m1_total_any_y2/mean_m3_m6_y2
			sum is_j_m3_m6_smooth_any[fw = n_m1]  
		} 
		else { 
			gen is_j_m3_m6_smooth_`clm'=m1_total_`clm'_y2/mean_m3_m6_y2_`clm'
			sum is_j_m3_m6_smooth_`clm'[fw = n_m1_`clm']  
		}
		local is_kink_avg_`clm' = r(mean) 
		mat table4[`row_num',7] =`is_kink_avg_`clm'' 

		
	restore
		bootstrap, reps(`bstraps'): ProbClaim_`clm'
		mat table4[`row_num',5] = _b[Decline1] 
		mat table4[`row_num',6] = _se[Decline1] 
	
	local row_num = `row_num' + 1 
}

mat list table4 
putexcel A1 = matrix(table4, names) using "${output}/Table4.xls", ///
		sheet("table4", replace) replace

}
