# R code for reporting results of counterfactural simulations
#Rprof(filename="run.Rprof",memory.profiling=TRUE)
library(RColorBrewer)
library(quantreg)
library(ggplot2)
library(data.table)
library(Rcpp)

dir.name <- "cara20"
paramfile <- "x.20"

#outDir <- "figures/3types/"
#paramfile <- "../estimation/baseline-pc3/smmEstCMA_2014-09-07_23.30.29.txt"
#cfgFile <- "../estimation/baseline-pc3/baseline3.cfg"

if (!exists("outDir") || is.na(outDir)) {
  outDir <- paste("figures/",dir.name,"/",sep="")
}
est.dir <- paste("../estimation/",dir.name,sep="")
if (!exists("paramfile") || is.na(paramfile)) {
  est.dir <- paste("../estimation/",dir.name,sep="")
  pfiles <- paste(est.dir,dir(est.dir,pattern="smmEstCMA(.+).txt"),sep="/")
  paramfile <- pfiles[order(pfiles,decreasing=TRUE)[1]]
  warning(sprintf("Using %s as paramfile",paramfile))
}
if (!exists("cfgFile") || is.na(cfgFile)) {
  est.dir <- paste("../estimation/",dir.name,sep="")
  cfgFile <- paste(est.dir,dir(est.dir,pattern="(.+).cfg$"),sep="/")
  if (length(cfgFile)>1) {
    stop("Cannot determine configuration file")
  }
}
#outDir <- "figures/nogapx/"


## Settings
#datafile <- "../c/data/p20.txt"
datafile <- "../estimation/data/medD-full-const.sl3"
nplanUse <- 500
probKeep <- 0.1
probKeep65 <- 0.1

#paramfile <- "../estimation/6types/smmEstCMA_2014-08-07_22.55.25.txt"
#cfgFile <- "../estimation/6types/6types.cfg"
#outDir <- "figures/6types/"
#cfgFile <- "../estimation/termVexcessJan/termVexcessJan.cfg"
#paramfile <- "../estimation/termVexcessJan/smmEstCMA_2014-08-23_12.47.20.txt"#smmEstCMA_2014-05-31_08.01.08.txt"
#cfgFile <- "../estimation/noCovariates/noCovariatess.cfg"
#paramfile <- "../estimation/noCovariates/smmEstCMA_2014-08-24_20.19.03.txt"
#outDir <- "figures/noCovariates/"
fdims <-1.5*c(5.03937, 3.77953) ## figure dimensions, =128mm x 96mm = dimensions of beamer slide
nsim <- 1 ## simulations per observation
nbin <- 75
colors <- brewer.pal(8,"Dark2") ## colors for figures
colors <- colors[c(1:3,5,8)]
forceFig <- TRUE ## recreate figures even if they already exist?




## read configuration
cfg.txt <- readLines(cfgFile)
cfg.txt <- gsub("true","TRUE",cfg.txt)
cfg.txt <- gsub("false","FALSE",cfg.txt)
cfg <- new.env()
eval(parse(text=cfg.txt),envir=cfg)

NX <- cfg$NX + 1
raEst <- cfg$estRiskAversion
nmix <- cfg$NMIX
nlambda <- cfg$nLambda
singleLambdaRatio <- cfg$singleLambdaRatio
terminalV <- cfg$terminalV
omegaThetaDep <- cfg$omegaThetaDep
if (is.null(omegaThetaDep)) {
  omegaThetaDep <- FALSE
}

## theme to mimic Liran's excel figures
fontFamily <- "Helvetica" ## change to Calibri if it's available
themeLE <- function(base_size=12, base_family=fontFamily) {
  theme_bw() %+replace%
  theme(axis.line = element_line(colour="grey"),
        panel.grid.minor = element_blank(),
        panel.background= element_blank(),
        panel.grid.major.x = element_blank(),
        panel.background = element_rect(fill="white",colour =NA),
        panel.border=element_blank(),
        text = element_text(family=base_family, size=base_size,
          face="plain", hjust=0.5, vjust=0.5, angle=0, lineheight=0.9,
          colour="black"),
        axis.text.x = element_text(angle=60),
        legend.position="bottom")
}
printDev <- function(filename, width=7, height=7, family=fontFamily) {
  cairo_pdf(filename,width=width,height=height,family=family)
  #pdf(filename,width,height)
}


source("param.R")
nbin <- 75

require(RColorBrewer)
require(ggplot2)

cflib <- "cf-dbg.so"
if (!is.loaded("cfsim")) {
  dyn.load(cflib)
}
rate <- t(matrix(c(1.0, 0.25, 1.0, 0.07,  ## std benefit
                   1.0, 0.25, 0.25, 0.07 ## filled gap
                   ),nrow=4))
dic <- t(matrix(c(275, 2510, 4050, ## std benefit
                  275, 2510, 4050 ## filled gap
                  ), nrow=3))
cfname <- c("standard benefit",
            "filled gap")

riskAversion <- 1e-4
tmp.file <- paste("x",as.numeric(Sys.time()),".tmp",sep="")
pm <- parameters(paramfile,tmp.file)

val.df <- function(riskAversion) {
  args <- list(xfile=tmp.file, dic=dic, rate=rate, cfgFile=cfgFile, riskAversion=riskAversion)
  result <- .Call("valueGrid",args)
  
  result <- data.frame(result)
  result$plan <- as.factor(result$plan)
  return(result)
}

ra <- c(0, 1e-9, 1e-8, 1e-6, 5e-6, 1e-5, 5e-5, 1e-4)

for(riskAversion in ra) {
  result <- val.df(riskAversion)
  result$riskAversion <- riskAversion
  if (riskAversion==0) {
    result$diff <- 0
    df <- result
    ra0 <- result
  } else {
    result$diff <- result$value-ra0$value
    df <- rbind(df,result)
  }
}

if (1) {
  time <-  51
  L <- 0
  m <- 1
  diff <- ggplot(data=subset(df, t==time & plan==0 & mix==m & lambda==L & riskAversion<1e-3
                  ),
                aes(x=cumCost, y=diff, colour=as.factor(riskAversion))) + geom_line()
  lev <- ggplot(data=subset(df, t==time & plan==0 & mix==m & lambda==L & riskAversion<1e-3
                  ),
                aes(x=cumCost, y=value, colour=as.factor(riskAversion))) + geom_line() 

  pdf("vplot.pdf",width=fdims[1],height=fdims[2])
  print(diff)
  print(lev)
  dev.off()
}

#sapply(mtq, function(x) { summary(df$value[df$maxThetaQ==max(maxThetaQ)] - df$value[df$maxThetaQ==x]) })
#sapply(nint, function(x) { summary(df$value2[df$nint==max(nint)] - df$value2[df$nint==x]) })

sdf <- subset(df, mix==m & plan==0 & riskAversion==0 & lambda==0)

# plot as function of time
if (0) {
  m <- 3
  fig <- ggplot(data=subset(df, mix==m & plan==0 & riskAversion==0 & lambda==0),
                aes(x=cumCost, y=value, colour=as.factor(t))) + geom_line() +
      scale_colour_discrete(h=c(170,230),l=seq(0,80,length.out=52))
  pdf("vplot.pdf",width=fdims[1],height=fdims[2])
  print(fig)
  dev.off()
}
  
unlink(tmp.file)
